@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: packet.c,v 1.3 1997/09/03 17:45:16 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: packet.c,v 1.4 1998/04/05 00:14:32 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -93,11 +93,20 @@ Reg	int	length;
 	ch1 = bufptr + cptr->count;
 	ch2 = buffer;
 #ifdef	ZIP_LINKS
+        if (cptr->flags & FLAGS_ZIPSTART)
+            {
+                if (*ch2 == '\n') /* also check \r ? */
+                    {
+                        ch2++;
+                        length--;
+                    }   
+                cptr->flags &= ~FLAGS_ZIPSTART;
+            }   
 	if (cptr->flags & FLAGS_ZIP)
 	    {
 		/* uncompressed buffer first */
 		zipped = length;
-		ch2 = unzip_packet(cptr, buffer, &zipped);
+		ch2 = unzip_packet(cptr, ch2, &zipped);
 		length = zipped;
 		zipped = 1;
 		if (length == -1)
@@ -169,13 +178,13 @@ Reg	int	length;
 				** contained PASS/SERVER and is now zipped!
 				** Ignore the '\n' that should be here.
 				*/
-				if (*ch2 == '\n')
+				zipped = length;
+				if (zipped > 0 && *ch2 == '\n')
 				    {
 					ch2++;
-					zipped = length - 1;
+					zipped--;
 				    }
-				else
-					zipped = length; /* impossible case? */
+				cptr->flags &= ~FLAGS_ZIPSTART;
 				ch2 = unzip_packet(cptr, ch2, &zipped);
 				length = zipped;
 				zipped = 1;
@@ -192,4 +201,3 @@ Reg	int	length;
 	cptr->count = ch1 - bufptr;
 	return r;
 }
-
