@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: send.c,v 1.19 1997/09/03 20:33:20 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: send.c,v 1.28 1998/09/09 12:25:26 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -166,7 +166,7 @@ int	len;
 				    poolsize);
 			istat.is_dbufmore++;
 		    }
-		else if (IsServer(to))
+		else if (IsServer(to) || IsService(to))
 			sendto_flag(SCH_ERROR,
 				"Max SendQ limit exceeded for %s: %d > %d",
 			   	get_client_name(to, FALSE),
@@ -177,7 +177,7 @@ int	len;
 			return dead_link(to, "Max Sendq exceeded");
 		    }
 #  else /* HUB */
-		if (IsServer(to))
+		if (IsService(to) || IsServer(to))
 			sendto_flag(SCH_ERROR,
 				"Max SendQ limit exceeded for %s: %d > %d",
 			   	get_client_name(to, FALSE),
@@ -285,7 +285,7 @@ tryagain:
 		if (DBufLength(&to->sendQ) > get_sendq(to))
 		    {
 #  ifdef HUB
-			if (IsServer(to) && CBurst(to))
+			if ((IsService(to) || IsServer(to)) && CBurst(to))
 			    {
 				aClass	*cl = to->serv->nline->class;
 
@@ -297,9 +297,9 @@ tryagain:
                                             poolsize);
 				istat.is_dbufmore++;
 			    }
-			else if (IsServer(to))
+			else if (IsServer(to) || IsService(to))
 #  else
-			if (IsServer(to))
+			if (IsServer(to) || IsService(to))
 #  endif
 			sendto_flag(SCH_ERROR,
 				"Max SendQ limit exceeded for %s: %d > %d",
@@ -706,6 +706,7 @@ void	sendto_channel_butone(aClient *one, aClient *from, aChannel *chptr, char *p
 
 	if (one != from && MyConnect(from) && IsRegisteredUser(from))
 	    {
+		/* useless junk? */
 #if ! USE_STDARG
 		sendto_prefix_one(from, from, pattern, p1, p2, p3, p4,
 				  p5, p6, p7, p8, p9, p10, p11);
@@ -829,6 +830,39 @@ void	sendto_serv_v(aClient *one, int ver, char *pattern, ...)
 	return;
 }
 
+#if ! USE_STDARG
+void	sendto_serv_notv(one, ver, pattern, p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11)
+aClient *one;
+int	ver;
+char	*pattern, *p1, *p2, *p3, *p4, *p5, *p6, *p7, *p8, *p9, *p10, *p11;
+#else
+void	sendto_serv_notv(aClient *one, int ver, char *pattern, ...)
+#endif
+{
+	Reg	int	i, len=0;
+	Reg	aClient *cptr;
+
+	for (i = fdas.highest; i >= 0; i--)
+		if ((cptr = local[fdas.fd[i]]) &&
+		    (!one || cptr != one->from) && !IsMe(cptr) &&
+		    ((cptr->serv->version & ver) == 0)) {
+			if (!len)
+			    {
+#if ! USE_STDARG
+				len = sendprep(pattern, p1, p2, p3, p4, p5,
+					       p6, p7, p8, p9, p10, p11);
+#else
+				va_list	va;
+				va_start(va, pattern);
+				len = vsendprep(pattern, va);
+				va_end(va);
+#endif
+			    }
+			(void)send_message(cptr, sendbuf, len);
+	}
+	return;
+}
+
 /*
  * sendto_common_channels()
  *
@@ -1090,6 +1124,64 @@ void	sendto_match_servs(aChannel *chptr, aClient *from, char *format, ...)
 			continue;
 		if (!BadPtr(mask) && match(mask, cptr->name))
 			continue;
+		if (chptr &&
+		    *chptr->chname == '!' && !(cptr->serv->version & SV_NJOIN))
+			continue;
+		if (!len)
+		    {
+#if ! USE_STDARG
+			len = sendprep(format, p1, p2, p3, p4, p5, p6, p7,
+				       p8, p9, p10, p11);
+#else
+			va_list	va;
+			va_start(va, format);
+			len = vsendprep(format, va);
+			va_end(va);
+#endif
+		    }
+		(void)send_message(cptr, sendbuf, len);
+	    }
+}
+
+#if ! USE_STDARG
+/*VARARGS*/
+void	sendto_match_servs_v(chptr, from, ver, format, p1, p2, p3, p4, p5, p6,
+			     p7, p8, p9, p10, p11)
+aChannel *chptr;
+aClient	*from;
+char	*format, *p1, *p2, *p3, *p4, *p5, *p6, *p7, *p8, *p9, *p10, *p11;
+int	ver;
+#else
+void	sendto_match_servs_v(aChannel *chptr, aClient *from, int ver,
+			     char *format, ...)
+#endif
+{
+	Reg	int	i, len=0;
+	Reg	aClient	*cptr;
+	char	*mask;
+
+	if (chptr)
+	    {
+		if (*chptr->chname == '&')
+			return;
+		if ((mask = (char *)rindex(chptr->chname, ':')))
+			mask++;
+	    }
+	else
+		mask = (char *)NULL;
+
+	for (i = fdas.highest; i >= 0; i--)
+	    {
+		if (!(cptr = local[fdas.fd[i]]) || (cptr == from) ||
+		    IsMe(cptr))
+			continue;
+		if (!BadPtr(mask) && match(mask, cptr->name))
+			continue;
+		if ((ver & cptr->serv->version) == 0)
+			continue;
+		if (chptr &&
+		    *chptr->chname == '!' && !(cptr->serv->version & SV_NJOIN))
+			continue;
 		if (!len)
 		    {
 #if ! USE_STDARG
@@ -1186,6 +1278,8 @@ void	sendto_ops_butone(aClient *one, aClient *from, char *pattern, ...)
 	bzero((char *)&sentalong[0], sizeof(int) * MAXCONNECTIONS);
 	for (cptr = client; cptr; cptr = cptr->next)
 	    {
+		if (IsService(cptr) || !IsRegistered(cptr))
+			continue;
 		if ((IsPerson(cptr) && !SendWallops(cptr)) || IsMe(cptr))
 			continue;
 		if (MyClient(cptr) && !(IsServer(from) || IsMe(from)))
@@ -1270,6 +1364,7 @@ static	SChan	svchans[SCH_MAX] = {
 	{ SCH_LOCAL,	"&LOCAL",	NULL },
 	{ SCH_SERVICE,	"&SERVICES",	NULL },
 	{ SCH_DEBUG,	"&DEBUG",	NULL },
+	{ SCH_AUTH,	"&AUTH",	NULL },
 };
 
 
