@@ -32,8 +32,8 @@ EXTERN int send_queued __P((aClient *to));
 #if ! USE_STDARG
 EXTERN int sendto_one();
 #else /* USE_STDARG */
-EXTERN int vsendto_one __P((aClient *to, char *pattern, va_list va);
-EXTERN int sendto_one __P((aClient *to, char *pattern, ...);
+EXTERN int vsendto_one (aClient *to, char *pattern, va_list va);
+EXTERN int sendto_one (aClient *to, char *pattern, ...);
 #endif /* USE_STDARG */
 #ifndef CLIENT_COMPILE
 EXTERN void flush_connections __P((int fd));
@@ -44,30 +44,35 @@ EXTERN void sendto_flog __P((aClient *cptr, char *msg, time_t duration,
 EXTERN void sendto_channel_butone();
 EXTERN void sendto_serv_butone();
 EXTERN void sendto_serv_v();
+EXTERN void sendto_serv_notv();
 EXTERN void sendto_common_channels();
 EXTERN void sendto_channel_butserv();
 EXTERN void sendto_match_servs();
+EXTERN void sendto_match_servs_v();
 EXTERN void sendto_match_butone();
 EXTERN void sendto_ops_butone();
 EXTERN void sendto_prefix_one();
 EXTERN void sendto_flag();
 #else /* USE_STDARG */
-EXTERN void sendto_channel_butone __P((aClient *one, aClient *from,
-				       aChannel *chptr, char *pattern, ...));
-EXTERN void sendto_serv_butone __P((aClient *one, char *pattern, ...));
-EXTERN void sendto_serv_v __P((aClient *one, int ver, char *pattern, ...));
-EXTERN void sendto_common_channels __P((aClient *user, char *pattern, ...));
-EXTERN void sendto_channel_butserv __P((aChannel *chptr, aClient *from,
-					char *pattern, ...));
-EXTERN void sendto_match_servs __P((aChannel *chptr, aClient *from,
-				    char *format, ...));
-EXTERN void sendto_match_butone __P((aClient *one, aClient *from, char *mask,
-				     int what, char *pattern, ...));
-EXTERN void sendto_ops_butone __P((aClient *one, aClient *from, char *pattern,
-				   ...));
-EXTERN void sendto_prefix_one __P((aClient *to, aClient *from, char *pattern,
-				   ...));
-EXTERN void sendto_flag __P((u_int chan, char *pattern, ...));
+EXTERN void sendto_channel_butone (aClient *one, aClient *from,
+				   aChannel *chptr, char *pattern, ...);
+EXTERN void sendto_serv_butone (aClient *one, char *pattern, ...);
+EXTERN void sendto_serv_v (aClient *one, int ver, char *pattern, ...);
+EXTERN void sendto_serv_notv (aClient *one, int ver, char *pattern, ...);
+EXTERN void sendto_common_channels (aClient *user, char *pattern, ...);
+EXTERN void sendto_channel_butserv (aChannel *chptr, aClient *from,
+				    char *pattern, ...);
+EXTERN void sendto_match_servs (aChannel *chptr, aClient *from,
+				char *format, ...);
+EXTERN void sendto_match_servs_v (aChannel *chptr, aClient *from, int ver,
+				char *format, ...);
+EXTERN void sendto_match_butone (aClient *one, aClient *from, char *mask,
+				 int what, char *pattern, ...);
+EXTERN void sendto_ops_butone (aClient *one, aClient *from, char *pattern,
+			       ...);
+EXTERN void sendto_prefix_one (aClient *to, aClient *from, char *pattern,
+			       ...);
+EXTERN void sendto_flag (u_int chan, char *pattern, ...);
 #endif /* USE_STDARG */
 #endif /* CLIENT_COMPILE */
 #undef EXTERN
