@@ -17,7 +17,7 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: hash.c,v 1.4 1997/09/03 17:45:49 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: hash.c,v 1.9 1998/05/25 20:44:20 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -85,9 +85,11 @@ int	*store;
 		hash <<= 1;
 		hash += hashtab[(int)ch];
 	}
-	*store = hash;
+	/*
 	if (hash < 0)
 		hash = -hash;
+	*/
+	*store = hash;
 	hash %= _HASHSIZE;
 	return (hash);
 }
@@ -109,6 +111,8 @@ int	*store;
 	Reg	int	i = 30;
 	Reg	u_int	hash = 5;
 
+	if (*name == '!')
+		name += 1 + CHIDLEN;
 	for (; (ch = *name) && --i; name++)
 	{
 		hash <<= 1;
@@ -136,7 +140,10 @@ int	size;
 	if (size < 4)
 		return size;
 
-	for ( ; ; size++)
+	if (size % 2 == 0)      /* Make sure it's odd because... */
+		size++;
+	
+	for ( ; ; size += 2)    /* ...no point checking even numbers - Core */
 	    {
 		failure = 0;
 		sq = (int)sqrt((double)size);
@@ -571,10 +578,10 @@ aClient *cptr;
 	for (;;)
 	    {
 		t--;
-		for (; t > server; t--)
+		for (; t >= server; t--)
 			if (*(t+1) == '.')
 				break;
-		if (*t == '*' || t == server)
+		if (t < server || *t == '*')
 			break;
 		ch = *t;
 		*t = '*';
@@ -595,6 +602,9 @@ aClient *cptr;
 
 /*
  * hash_find_channel
+ *
+ * If the name doesn't begin with #, or & or -, then we're looking for
+ * -????name instead of a real match.
  */
 aChannel	*hash_find_channel(name, chptr)
 char	*name;
@@ -602,13 +612,20 @@ aChannel *chptr;
 {
 	Reg	aChannel	*tmp, *prv = NULL;
 	Reg	aHashEntry	*tmp3;
-	u_int	hashv, hv;
+	u_int	hashv, hv, exact;
 
 	hashv = hash_channel_name(name, &hv);
 	tmp3 = &channelTable[hashv];
 
+	if (IsChannelName(name))
+		exact = 1;
+	else
+		exact = 0;
 	for (tmp = (aChannel *)tmp3->list; tmp; prv = tmp, tmp = tmp->hnextch)
-		if (hv == tmp->hashv && mycmp(name, tmp->chname) == 0)
+		if (hv == tmp->hashv &&
+		    ((exact == 1 && mycmp(name, tmp->chname) == 0) ||
+		     (exact == 0 && *tmp->chname == '!' &&
+		      mycmp(name, tmp->chname + CHIDLEN + 1) == 0)))
 		    {
 			chhits++;
 			if (prv)
@@ -890,3 +907,5 @@ char	*parv[];
 	return 2;
 #endif
 }
+
+
